/**
 * VolCB control by Arduino
 * by alexcp <https://hifiocean.com>
 * 
 * Demonstrates Arduino controlling Hans Polak's volume control board (VolCB)
 * for Bruno Putzeys' Purist Balanced Preamp (BPPBP). Uses two rotary encoders
 * to control input selection and volume.
 * 
 * Uses Wire (part of Arduino distibution) and 
 * Encoder (https://www.pjrc.com/teensy/td_libs_Encoder.html) libraries.
 * 
 * This code is in the public domain.
 *
 * Connections
 * 
 * Encoder to select input:  A to digital pin 2, B to digital pin 3, common to GND
 * Encoder to select volume: A to digital pin 4, B to digital pin 5, common to GND
 * VolCB: GND to Arduino's GND, SCL to SCL (analog pin A5), SDA to SDA (analog pin A4)
 * 
 * Theory of operation
 * 
 * VolCB carries two PCF8574/PCF8574A 8-bit I/O expanders, controller via I2C bus.
 * PCF8574 and PCF8574A are identical except for their I2C address.
 * On the diyAudio GB boards, PCF8574A were installed, and this is what ths sketch
 * assumes by default. If your have PCF8574A, uncomment two #define lines below.
 * 
 * One PCF8574/PCF8574A, with the (7-bit) address 0x20 (PCF8574) or
 * 0x38 (PCF8574A), controls volume as follows.
 * 
 * Bit P0 controls 1dB attenuator (i.e. setting bit P0 to LOW switches
 * the attenuator OFF), P1 controls 2dB attenuator, and so on; P5 controls
 * 32dB attenuator. Bits P6 and P7 are not connected.
 * 
 * The other PCF8574/PCF8574A, with the (7-bit) address 0x24 (PCF8574) or
 * 0x3C (PCF8574A), controls the input selector as follows. 
 * 
 * - Bit P0 controls the (only) selector relay on the BPPBP board, via
 *   helper relay Re7 or, if the 4-input selector board is installed,
 *   via Re1 on the 4-input board.
 * - In addition, with the 4-input selector board, bits P1..P3 control
 *   additional relays: P1 controls Re2/Re3, P2 - Re4/Re5, P3 - Re6/Re7.
 *   Bits P4 and P5 are not used but made available for external
 *   connections. Bits P6 and P7 are not connected.
 * - Setting each bit to LOW energizes the respective relay(s)
 * 
 * PCF8574/PCF8574A is controlled via I2C bus (Arduino's Wire library)
 * like this:
 *   Wire.beginTransmission( 0x38 ); // transmit to PCF8574A (0x38)
 *   Wire.write( byte( 0x12 ) );     // select 0x12 = 18dB attenuation  
 *   Wire.endTransmission();
 */

// uncomment the following #define if PCF8574 is installed as U1 on VolCB
// leave it commented out if PCF8574A is installed
// #define VOLCB_PCF8574

// uncomment the following #define if PCF8574 is installed as U2 on VolCB
// leave it commented out if PCF8574A is installed
// #define FOURINPUT_PCF8574

#include "Wire.h"    // include Wire library
#include "Encoder.h" // include Encoder library

#ifdef VOLCB_PCF8574
#define VOLCB_ADDR ( 0x20 )
#else
#define VOLCB_ADDR ( 0x38 )
#endif

#ifdef FOURINPUT_PCF8574
#define FOURINPUT_ADDR ( 0x24 )
#else
#define FOURINPUT_ADDR ( 0x3C )
#endif

// declare encoders
Encoder knobInput( 2, 3 );  // input selector
Encoder knobVolume( 4, 5 ); // volume control

// get user input from the rotary encoder
byte getAttenuation() {
  long val = knobVolume.read();  // read encoder's position
  val = constrain( val, 0, 63 ); // limit to appropriate values for attenuation
  knobVolume.write( val );       // update encoder with constrained value
  return val;
}

void setAttenuation( byte attenuation ) {
  Wire.beginTransmission( VOLCB_ADDR );          // transmit to U1
  Wire.write( constrain( attenuation, 0, 63 ) ); // limit attenuation to the appropriate range
  Wire.endTransmission();
}

// get user input from the rotary encoder
byte getInput() { 
  long val = knobInput.read();  // read encoder's position
  val = constrain( val, 0, 3 ); // limit to appropriate values for attenuation
  knobInput.write( val );       // update encoder with constrained value
  return val;
}

void setInput( byte input ) {
  Wire.beginTransmission( FOURINPUT_ADDR );             // transmit to U2
  Wire.write( ~byte( 1 << constrain( input, 0, 3 ) ) ); // set just one bit from [0,1,2,3] to LOW
  Wire.endTransmission();
}

byte currentAttenuation = 0x3F; // current attenuation, initially max attenuation
byte currentInput  = 0;         // currently selected input, initially input 0

void setup() {
  Wire.begin();                         // join I2C bus
  setAttenuation( currentAttenuation ); // set the initial values
  setInput( currentInput );
}

void loop() {
  byte newInput = getInput(); // get user input
  if( newInput != currentInput ) { // check if the new value is different
    currentInput = newInput;       // remember new value
    setInput( currentInput );      // update input selector
  }

  byte newAttenuation = getAttenuation(); // get user input
  if( newAttenuation != currentAttenuation ) { // check if the new value is different
    currentAttenuation = newAttenuation;       // remember new value
    setAttenuation( currentAttenuation );      // update volume control
  }
}
